/*

How to do an index review - Workload
April 2010
Mike DeFehr
www.mikedefehr.com
winnipeg.sqlpass.org

*/

--use the TestQT database from Intro to Query Tuning
USE TestQT
GO

--create a standard set of non-clustered indexes on each foreign key
CREATE INDEX ix_CustomerShipTo_CustomerID ON [CustomerShipTo] (CustomerID)
CREATE INDEX ix_Product_ProductCategoryID ON [Product] (ProductCategoryID)
CREATE INDEX ix_Product_CustomerID ON [Product] (CustomerID)
CREATE INDEX ix_ProductOrder_CustomerID ON [ProductOrder] (CustomerID)
CREATE INDEX ix_ProductOrderLine_ProductOrderID ON [ProductOrderLine] (ProductOrderID)
CREATE INDEX ix_ProductOrderLine_ProductID ON [ProductOrderLine] (ProductID)
CREATE INDEX ix_ProductOrderDelivery_ProductOrderLineID ON [ProductOrderDelivery] (ProductOrderLineID)
CREATE INDEX ix_ProductOrderDelivery_CustomerShipToID ON [ProductOrderDelivery] (CustomerShipToID)
GO

IF OBJECT_ID('uspProductOrder') IS NOT NULL DROP PROC uspProductOrder
GO
CREATE PROC uspProductOrder
@InProductOrderID int
AS
SELECT	c.CustomerName, o.CustomerPOID, o.OrderDate, l.ProductID, d.OrderQty, d.ProductPrice, d.DeliveryDate
FROM ProductOrder o
	JOIN Customer c ON o.CustomerID = c.CustomerID
	JOIN ProductOrderLine l ON o.ProductOrderID = l.ProductOrderID
	JOIN ProductOrderDelivery d ON l.ProductOrderLineID = d.ProductOrderLineID
WHERE o.ProductOrderID = @InProductOrderID	
GO

IF OBJECT_ID('uspProductLookupByID') IS NOT NULL DROP PROC uspProductLookupByID
GO
CREATE PROC uspProductLookupByID
@InProductID int
AS
SELECT	p.ProductID, p.ProductName, p.ProductPrice, pc.ProductCategoryName
FROM Product p
	JOIN ProductCategory pc ON p.ProductCategoryID = pc.ProductCategoryID
WHERE p.ProductID = @InProductID
GO

IF OBJECT_ID('uspProductLookupByName') IS NOT NULL DROP PROC uspProductLookupByName
GO
CREATE PROC uspProductLookupByName
@InProductName varchar(255)
AS
SELECT	p.ProductID, p.ProductName, p.ProductPrice, pc.ProductCategoryName
FROM Product p
	JOIN ProductCategory pc ON p.ProductCategoryID = pc.ProductCategoryID
WHERE p.ProductName = @InProductName
GO

IF OBJECT_ID('uspProductListByCategory') IS NOT NULL DROP PROC uspProductListByCategory
GO
CREATE PROC uspProductListByCategory
@InCategoryName varchar(255)
AS
SELECT	p.ProductID, p.ProductName, p.ProductPrice, pc.ProductCategoryName
FROM Product p
	JOIN ProductCategory pc ON p.ProductCategoryID = pc.ProductCategoryID
WHERE pc.ProductCategoryName = @InCategoryName
GO

IF OBJECT_ID('uspOrderShipToLookup') IS NOT NULL DROP PROC uspOrderShipToLookup
GO
CREATE PROC uspOrderShipToLookup
@InProductOrderID int
AS
SELECT	l.ProductID, d.OrderQty, d.DeliveryDate, st.ShipToAddress, st.ShipToInformation
FROM ProductOrder o
	JOIN Customer c ON o.CustomerID = c.CustomerID
	JOIN ProductOrderLine l ON o.ProductOrderID = l.ProductOrderID
	JOIN ProductOrderDelivery d ON l.ProductOrderLineID = d.ProductOrderLineID
	JOIN CustomerShipTo st ON d.CustomerShipToID = st.CustomerShipToID
WHERE o.ProductOrderID = @InProductOrderID	
GO

IF OBJECT_ID('uspCustomerLookup') IS NOT NULL DROP PROC uspCustomerLookup
GO
CREATE PROC uspCustomerLookup
@InCustomerName varchar(255)
AS
SELECT	c.CustomerID, c.CustomerName, c.CustomerInformation, st.ShipToAddress
FROM Customer c
	JOIN CustomerShipTo st ON c.CustomerID = st.CustomerID
WHERE c.CustomerName = @InCustomerName
GO

IF OBJECT_ID('uspNewOrder') IS NOT NULL DROP PROC uspNewOrder
GO
CREATE PROC uspNewOrder
@MaxCustID int --(SELECT MAX(customerid) FROM Customer
AS
--EXEC uspNewOrder 50,100
SET NOCOUNT ON
DECLARE @OrdID int, @CurCustID int, @ErrCount int = 0, @CurOrdDate date, @RandNum float
DECLARE @OrderLines TABLE (ProductOrderLineID int, ExpandBy float)

WHILE @ErrCount >=0 AND @ErrCount < 100
BEGIN
	BEGIN TRY
	
	SET @RandNum = RAND(CHECKSUM(NEWID()))

	SET @CurCustID = CAST( @RandNum * @MaxCustID AS int) + 1
	
	SELECT TOP 1 @CurOrdDate = ISNULL(OrderDate,'2008-01-01') FROM ProductOrder ORDER BY ProductOrderID DESC
	
	IF @RandNum * 1000 < 5 SET @CurOrdDate = DATEADD(DAY,1,@CurOrdDate)

	INSERT ProductOrder (CustomerID,OrderDate,OrderInformation)
	VALUES (@CurCustID,@CurOrdDate,CAST(NEWID() AS CHAR(36)))
	SET @OrdID = SCOPE_IDENTITY();
	
	UPDATE ProductOrder SET CustomerPOID = CAST(ProductOrderID as varchar) WHERE ProductOrderID = @OrdID
		
	SET @ErrCount = -1
	
	END TRY
	BEGIN CATCH
		SET @ErrCount += 1
		PRINT @ErrCount
		IF @ErrCount > 10 
		BEGIN
			RAISERROR('ProductOrder Insert Unsuccessful',16,1)
			RETURN
		END
	END CATCH
END;


WITH prods AS
(
	SELECT	CAST(RAND(CHECKSUM(NEWID())) * 1000 AS int) + 1 AS RandNum,
			ProductID
	FROM Product
	WHERE CustomerID = @CurCustID
)
INSERT @OrderLines
SELECT ProductOrderLineID, RAND(CHECKSUM(NEWID()))
FROM (
		INSERT ProductOrderLine(ProductOrderID,ProductID,OrderLineInformation)
		OUTPUT inserted.ProductOrderLineID
		SELECT  @OrdID, prods.ProductID, CAST(NEWID() AS NCHAR(200))
		FROM prods
		WHERE RandNum % (CAST(RAND(CHECKSUM(NEWID())) * 10 AS int) + 1) = 0
	) a;
	
WITH shiptos AS
(
	SELECT	CustomerShipToID,
			RAND(CHECKSUM(NEWID())) AS RandNum
	FROM CustomerShipTo
	WHERE CustomerID = @CurCustID
)
INSERT ProductOrderDelivery (ProductOrderLineID,CustomerShipToID,OrderQty,ProductPrice,DeliveryDate,OrderDeliveryInformation)
SELECT	ol.ProductOrderLineID,
		st.CustomerShipToID,
		CAST(RAND(CHECKSUM(NEWID())) * 200 AS int) + 1,
		p.ProductPrice + CAST(RAND(CHECKSUM(NEWID())) * 10 AS decimal(19,4)) - 5,
		DATEADD(day,CAST(RAND(CHECKSUM(NEWID())) * 20 AS int) + 3,@CurOrdDate),
		CAST(NEWID() AS NCHAR(200))
FROM @OrderLines ol
	JOIN ProductOrderLine l ON ol.ProductOrderLineID = l.ProductOrderLineID
	JOIN Product p ON l.ProductID = p.ProductID
	CROSS APPLY	(SELECT TOP (ol.ExpandBy * 100) PERCENT CustomerShipToID
				 FROM shiptos
				 ORDER BY RandNum) st
GO

IF OBJECT_ID('uspUpdateCustomer') IS NOT NULL DROP PROC uspUpdateCustomer
GO
CREATE PROC uspUpdateCustomer
@InCustomerID int
AS
UPDATE Customer SET CustomerInformation = CAST(NEWID() AS NCHAR(100)) WHERE CustomerID = @InCustomerID
GO
IF OBJECT_ID('uspUpdateProduct') IS NOT NULL DROP PROC uspUpdateProduct
GO
CREATE PROC uspUpdateProduct
@InProductID int
AS
UPDATE Product SET ProductInformation = CAST(NEWID() AS NCHAR(1000)) WHERE ProductID = @InProductID
GO
IF OBJECT_ID('uspUpdateProductOrder') IS NOT NULL DROP PROC uspUpdateProductOrder
GO
CREATE PROC uspUpdateProductOrder
@InProductOrderID int
AS
UPDATE ProductOrder SET OrderInformation = CAST(NEWID() AS CHAR(36)) WHERE ProductOrderID = @InProductOrderID
GO
IF OBJECT_ID('uspUpdateProductOrderLine') IS NOT NULL DROP PROC uspUpdateProductOrderLine
GO
CREATE PROC uspUpdateProductOrderLine
@InProductOrderID int
AS
DECLARE @OrderLineID int

SELECT TOP 1 @OrderLineID = ProductOrderLineID
FROM ProductOrderLine
WHERE ProductOrderID = @InProductOrderID
ORDER BY RAND(CHECKSUM(NEWID()))

UPDATE ProductOrderLine SET OrderLineInformation = CAST(NEWID() AS NCHAR(200))
WHERE ProductOrderID = @OrderLineID
GO

IF OBJECT_ID('uspUpdateProductOrderDelivery') IS NOT NULL DROP PROC uspUpdateProductOrderDelivery
GO
CREATE PROC uspUpdateProductOrderDelivery
@InProductOrderID int
AS
DECLARE @OrderDeliveryID int

SELECT TOP 1 @OrderDeliveryID = ProductOrderDeliveryID
FROM ProductOrderLine l
	JOIN ProductOrderDelivery d ON l.ProductOrderLineID = d.ProductOrderLineID
WHERE l.ProductOrderID = @InProductOrderID
ORDER BY RAND(CHECKSUM(NEWID()))

UPDATE ProductOrderDelivery SET OrderDeliveryInformation = CAST(NEWID() AS NCHAR(200))
WHERE ProductOrderDeliveryID = @OrderDeliveryID
GO

IF OBJECT_ID('uspNewProduct') IS NOT NULL DROP PROC uspNewProduct
GO
CREATE PROC uspNewProduct
AS

DECLARE @CustomerID int, @CategoryID int, @NewProd int

SELECT TOP 1 @CustomerID = CustomerID
FROM Customer
ORDER BY RAND(CHECKSUM(NEWID()))

SELECT TOP 1 @CategoryID = ProductCategoryID
FROM ProductCategory
ORDER BY RAND(CHECKSUM(NEWID()))

INSERT Product (ProductPrice, ProductCategoryID, CustomerID, ProductInformation)
VALUES (CAST(RAND(CHECKSUM(NEWID()))*10+10 AS decimal(19,4)),@CategoryID,@CustomerID,CAST(NEWID() AS NCHAR(1000)))
SET @NewProd = SCOPE_IDENTITY()

UPDATE Product SET ProductName = 'Product ' + CAST(@NewProd AS varchar) WHERE ProductID = @NewProd
GO

IF OBJECT_ID('uspExecuteWorkload') IS NOT NULL DROP PROC uspExecuteWorkload
GO
CREATE PROC uspExecuteWorkload
@NumMin int
AS
SET NOCOUNT ON
DECLARE @CustomerLookup TABLE
(CustomerID int, CustomerName varchar(255), CustomerInformation nchar(100), ShipToAddress varchar(100))
DECLARE @OrderShipToLookup TABLE
(ProductID int, OrderQty int, DeliveryDate datetime, ShiptoAddress varchar(100), ShipToInformation nchar(1000))
DECLARE @ProductListLookup TABLE
(ProductID int, ProductName varchar(255), ProductPrice money, ProductCategoryName varchar(255))
DECLARE @ProductOrder TABLE
(CustomerName varchar(255), CustomerPOID varchar(100), OrderDate datetime, ProductID int, OrderQty int, ProductPrice money, DeliveryDate datetime)

DECLARE @StopTime datetime, @ProcIndex smallint, @CurID int, @TextParam varchar(255), @IterCount int
DECLARE @MaxCustID int, @MaxOrderID int, @MaxCategoryID int, @MaxProductID int

SET @IterCount = 1

SELECT @MaxCustID = MAX(CustomerID) FROM Customer
SELECT @MaxOrderID = MAX(ProductOrderID) FROM ProductOrder
SELECT @MaxCategoryID = MAX(ProductCategoryID) FROM ProductCategory
SELECT @MaxProductID = MAX(ProductID) FROM Product


IF @MaxOrderID IS NULL
BEGIN
	exec uspNewOrder @MaxCustID
	SET @MaxOrderID = 1
END


SET @StopTime = DATEADD(minute,@NumMin,GETDATE())
WHILE GETDATE() < @StopTime
BEGIN
	SET @IterCount += 1
	SET @ProcIndex = CAST(RAND(CHECKSUM(NEWID())) * 1000 AS smallint) + 1
	IF @IterCount % 1000 = 0
		RAISERROR('IterCount: %i',0,0,@IterCount)
	IF @Procindex BETWEEN 1 AND 500
	BEGIN
		SET @CurID = CAST(RAND(CHECKSUM(NEWID())) * @MaxCustID AS int) + 1
		SET @TextParam = 'Customer ' + CAST(@CurID AS varchar)
		INSERT @CustomerLookup
		EXEC uspCustomerLookup @TextParam
		IF @@ROWCOUNT = 0 PRINT 'Warning - no rows for uspCustomerLookup (' + ISNULL(@TextParam,'<null') + ')'
		DELETE FROM @CustomerLookup
	END
	IF @Procindex BETWEEN 1 AND 100
	BEGIN
		SET @CurID = CAST(RAND(CHECKSUM(NEWID())) * @MaxOrderID AS int) + 1
		INSERT @OrderShipToLookup
		EXEC uspOrderShipToLookup @CurID
		IF @@ROWCOUNT = 0 PRINT 'Warning - no rows for uspOrderShipToLookup (' + ISNULL(CAST(@CurID as varchar),'<null>') + ')'
		DELETE FROM @OrderShipToLookup
	END
	IF @Procindex BETWEEN 1 AND 50
	BEGIN
		SET @CurID = CAST(RAND(CHECKSUM(NEWID())) * @MaxCategoryID AS int) + 1
		SET @TextParam = 'Category ' + CAST(@CurID AS varchar)
		INSERT @ProductListLookup
		EXEC uspProductListByCategory @TextParam
		IF @@ROWCOUNT = 0 PRINT 'Warning - no rows for uspProductListByCategory (' + + ISNULL(@TextParam,'<null') + + ')'
		DELETE FROM @ProductListLookup
	END
	IF @Procindex BETWEEN 1 AND 400
	BEGIN
		SET @CurID = CAST(RAND(CHECKSUM(NEWID())) * @MaxProductID AS int) + 1
		SET @TextParam = 'Product ' + CAST(@CurID AS varchar)
		INSERT @ProductListLookup
		EXEC uspProductLookupByName @TextParam
		IF @@ROWCOUNT = 0 PRINT 'Warning - no rows for uspProductLookupByName (' + + ISNULL(@TextParam,'<null') + + ')'
		DELETE FROM @ProductListLookup
	END
	IF @Procindex BETWEEN 1 AND 1000
	BEGIN
		SET @CurID = CAST(RAND(CHECKSUM(NEWID())) * @MaxProductID AS int) + 1
		INSERT @ProductListLookup
		EXEC uspProductLookupByID @CurID
		IF @@ROWCOUNT = 0 PRINT 'Warning - no rows for uspProductLookupByID (' + ISNULL(CAST(@CurID as varchar),'<null>') + ')'
		DELETE FROM @ProductListLookup
	END
	IF @Procindex BETWEEN 1 AND 650
	BEGIN
		SET @CurID = CAST(RAND(CHECKSUM(NEWID())) * @MaxOrderID AS int) + 1
		INSERT @ProductOrder
		EXEC uspProductOrder @CurID
		IF @@ROWCOUNT = 0 PRINT 'Warning - no rows for uspProductOrder (' + ISNULL(CAST(@CurID as varchar),'<null>') + ')'
		DELETE FROM @ProductOrder
	END
	IF @Procindex BETWEEN 1 AND 550
	BEGIN
		EXEC uspNewOrder @MaxCustID
		SET @MaxOrderID += 1
	END
	IF @Procindex BETWEEN 1 AND 5
	BEGIN
		SET @CurID = CAST(RAND(CHECKSUM(NEWID())) * @MaxCustID AS int) + 1
		EXEC uspUpdateCustomer @CurID
	END
	IF @Procindex BETWEEN 1 AND 20
	BEGIN
		SET @CurID = CAST(RAND(CHECKSUM(NEWID())) * @MaxOrderID AS int) + 1
		EXEC uspUpdateProductOrder @CurID
	END
	IF @Procindex BETWEEN 1 AND 50
	BEGIN
		SET @CurID = CAST(RAND(CHECKSUM(NEWID())) * @MaxProductID AS int) + 1
		EXEC uspUpdateProduct @CurID
	END
	IF @Procindex BETWEEN 1 AND 30
	BEGIN
		SET @CurID = CAST(RAND(CHECKSUM(NEWID())) * @MaxOrderID AS int) + 1
		EXEC uspUpdateProductOrderLine @CurID
	END
	IF @Procindex BETWEEN 1 AND 50
	BEGIN
		SET @CurID = CAST(RAND(CHECKSUM(NEWID())) * @MaxOrderID AS int) + 1
		EXEC uspUpdateProductOrderDelivery @CurID
	END
	IF @Procindex BETWEEN 1 AND 10
	BEGIN
		EXEC uspNewProduct
	END
END
PRINT '***FINAL***'
RAISERROR('IterCount: %i ',0,0,@IterCount)
GO